@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2008 Participatory Culture Foundation
+# Copyright (C) 2005-2009 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,17 +36,10 @@ import zipfile
 import tarfile
 import plistlib
 import datetime
-import platform
 import subprocess
 
 from glob import glob
 
-# =============================================================================
-# Check Python version
-# =============================================================================
-
-PYTHON_VERSION = platform.python_version()[0:3]
-
 # =============================================================================
 # Find the top of the source tree and set the search path accordingly
 # =============================================================================
@@ -57,7 +50,7 @@ ROOT_DIR = os.path.normpath(ROOT_DIR)
 
 PORTABLE_DIR = os.path.join(ROOT_DIR, 'portable')
 PLATFORM_DIR = os.path.join(ROOT_DIR, 'platform', 'osx')
-PLATFORM_PACKAGE_DIR = os.path.join(PLATFORM_DIR, 'platform')
+PLATFORM_PACKAGE_DIR = os.path.join(PLATFORM_DIR, 'plat')
 
 OS_INFO = os.uname()
 OS_VERSION = int(OS_INFO[2][0])
@@ -65,11 +58,14 @@ OS_VERSION = int(OS_INFO[2][0])
 if OS_VERSION == 9:
     SANDBOX_ROOT_DIR = os.path.normpath(os.path.normpath(os.path.join(ROOT_DIR, '..')))
     SANDBOX_DIR = os.path.join(SANDBOX_ROOT_DIR, 'sandbox')
-    PYTHON_LIB = os.path.join("/", "System", "Library", "Frameworks", "Python.framework", "Versions", "Current", "Python")
-    sys.path.insert(0, os.path.join(SANDBOX_DIR, 'lib', 'python2.5', 'site-packages'))
+    PYTHON_ROOT = os.path.join(SANDBOX_DIR, "Library", "Frameworks", "Python.framework", "Versions", "Current")
+    PYTHON_LIB = os.path.join(PYTHON_ROOT, "Python")
+    PYTHON_VERSION = "2.5"
+    sys.path.insert(0, os.path.join(PYTHON_ROOT, 'lib', 'python2.5', 'site-packages'))
 else:
     SANDBOX_DIR = "/usr/local"
     PYTHON_LIB = os.path.join("/", "Library", "Frameworks", "Python.framework", "Versions", "Current", "Python")
+    PYTHON_VERSION = "2.4"
 
 # =============================================================================
 # Look for the Boost library in various common places.
@@ -87,12 +83,12 @@ BOOST_INCLUDE_DIR = None
 BOOST_VERSION = None
 
 for searchDir in (SANDBOX_DIR, '/usr/local', '/opt/local', '/sw'):
-    libItems = glob(os.path.join(searchDir, 'lib/libboost_python-1_3*.dylib'))
-    incItems = glob(os.path.join(searchDir, 'include/boost-1_3*/'))
-    if len(libItems) == 1 and len(incItems) == 1:
+    libItems = glob(os.path.join(searchDir, 'lib/libboost_python*-1_35.*'))
+    incItems = glob(os.path.join(searchDir, 'include/boost-1_35/'))
+    if len(libItems) >= 1 and len(incItems) >= 1:
         BOOST_LIB_DIR = os.path.dirname(libItems[0])
         BOOST_INCLUDE_DIR = incItems[0]
-        match = re.search(r'libboost_python-(.*)\.dylib', libItems[0])
+        match = re.search(r'libboost_python-(.*)\..*', libItems[0])
         BOOST_VERSION = match.groups()[0]
         break
 
@@ -110,15 +106,11 @@ else:
 #   isysroot flag.
 # =============================================================================
 
-if OS_VERSION < 9:
-    BOOST_TYPE = ""
-else:
-    BOOST_TYPE = "-mt"
-
-BOOST_PYTHON_LIB = os.path.join(SANDBOX_DIR, "lib", "libboost_python%s-%s.a" % (BOOST_TYPE, BOOST_VERSION))
-BOOST_FILESYSTEM_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_filesystem%s-%s.a' % (BOOST_TYPE, BOOST_VERSION))
-BOOST_DATETIME_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_date_time%s-%s.a' % (BOOST_TYPE, BOOST_VERSION))
-BOOST_THREAD_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_thread%s-%s.a' % (BOOST_TYPE, BOOST_VERSION))
+BOOST_PYTHON_LIB = os.path.join(SANDBOX_DIR, "lib", "libboost_python-%s.a" % BOOST_VERSION)
+BOOST_FILESYSTEM_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_filesystem-%s.a' % BOOST_VERSION)
+BOOST_DATETIME_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_date_time-%s.a' % BOOST_VERSION)
+BOOST_THREAD_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_thread-%s.a' % BOOST_VERSION)
+BOOST_SYSTEM_LIB = os.path.join(SANDBOX_DIR, "lib", 'libboost_system-%s.a' % BOOST_VERSION)
 
 # =============================================================================
 # Only now may we import things from the local sandbox and our own tree
@@ -137,7 +129,6 @@ import portable
 sys.modules['miro'] = portable
 
 from miro import util
-from miro.frontends.html import template_compiler
 
 # =============================================================================
 # Utility function used to extract stuff from the binary kit
@@ -172,13 +163,13 @@ def extract_binaries(source, target, force=True):
 class Config (object):
     
     def __init__(self, path, themePath=None):
-        self.config = util.readSimpleConfigFile(path)
+        self.config = util.read_simple_config_file(path)
         self.themeDir = os.path.join(ROOT_DIR, 'platform', 'osx', 'build', 'theme')
         self.themeConfig = None
         if themePath is not None:
             self.extract_theme_content(themePath, self.themeDir)
             themeConfigPath = os.path.join(self.themeDir, "app.config")
-            self.themeConfig = util.readSimpleConfigFile(themeConfigPath)
+            self.themeConfig = util.read_simple_config_file(themeConfigPath)
         elif os.path.exists(self.themeDir):
             shutil.rmtree(self.themeDir)
         
@@ -196,11 +187,14 @@ class Config (object):
                     extract = False
                     break            
             if extract:
-                data = themeArchive.read(entry)
                 path = os.path.join(target, entry)
-                f = open(path, "wt")
-                f.write(data)
-                f.close()
+                if entry.endswith('/'):
+                    os.makedirs(path)
+                else:
+                    data = themeArchive.read(entry)
+                    f = open(path, "wt")
+                    f.write(data)
+                    f.close()
         themeArchive.close()
     
     def get_icon_file(self):
@@ -256,7 +250,7 @@ class MiroBuild (py2app):
 
     def setup_config(self):
         # Get subversion revision information.
-        revision = util.queryRevision(ROOT_DIR)
+        revision = util.query_revision(ROOT_DIR)
         if revision is None:
             revisionURL = 'unknown'
             revisionNum = '0000'
@@ -293,7 +287,9 @@ class MiroBuild (py2app):
         self.distribution.ext_modules.append(self.get_idletime_ext())
         self.distribution.ext_modules.append(self.get_keychain_ext())
         self.distribution.ext_modules.append(self.get_qtcomp_ext())
-        self.distribution.ext_modules.append(self.get_database_ext())
+        self.distribution.ext_modules.append(self.get_growl_ext())
+        self.distribution.ext_modules.append(self.get_growl_image_ext())
+        self.distribution.ext_modules.append(self.get_shading_ext())
         self.distribution.ext_modules.append(self.get_sorts_ext())
         self.distribution.ext_modules.append(self.get_fasttypes_ext())
         self.distribution.ext_modules.append(self.get_libtorrent_ext())
@@ -303,26 +299,27 @@ class MiroBuild (py2app):
             'miro.dl_daemon',
             'miro.dl_daemon.private',
             'miro.frontends',
-            'miro.frontends.html',
-            'miro.frontends.html.compiled_templates',
-            'miro.platform',
-            'miro.platform.frontends',
-            'miro.platform.frontends.html',
-            'miro.platform.renderers',
+            'miro.frontends.widgets',
+            'miro.plat',
+            'miro.plat.frontends',
+            'miro.plat.frontends.widgets'
         ]
-        self.includes.add('miro.frontends.html.compiled_templates')
 
         self.distribution.package_dir = {
             'miro': PORTABLE_DIR,
-            'miro.platform': PLATFORM_PACKAGE_DIR,
+            'miro.plat': PLATFORM_PACKAGE_DIR,
         }
         
         self.iconfile = self.config.get_icon_file()
         
         excludedResources = ['.svn', '.DS_Store']
-        resourceFiles = [os.path.join('Resources', x) for x in os.listdir('Resources') if x not in excludedResources]
-        resourceFiles.append('qt_extractor.py')
-        self.resources = resourceFiles
+        self.resources = [
+            os.path.join('Resources-Widgets', 'MainMenu.nib'),
+            os.path.join('Resources-Widgets', 'OverlayPalette.nib'),
+            os.path.join('Resources-Widgets', 'Credits.html'),
+            'qt_extractor.py'
+        ]
+        self.resources.extend(glob(os.path.join('Resources-Widgets', '*.png')))
     
     def setup_options(self):
         self.bundleRoot = os.path.join(self.dist_dir, '%s.app/Contents' % self.config.get('shortAppName'))
@@ -334,21 +331,32 @@ class MiroBuild (py2app):
     def get_idletime_ext(self):
         idletime_src = glob(os.path.join(ROOT_DIR, 'platform', 'osx', 'modules', 'idletime.c'))
         idletime_link_args = ['-framework', 'CoreFoundation']
-        return Extension("miro.platform.idletime", sources=idletime_src, extra_link_args=idletime_link_args)
+        return Extension("miro.plat.idletime", sources=idletime_src, extra_link_args=idletime_link_args)
     
     def get_keychain_ext(self):
         keychain_src = glob(os.path.join(ROOT_DIR, 'platform', 'osx', 'modules', 'keychain.c'))
         keychain_link_args = ['-framework', 'Security']
-        return Extension("miro.platform.keychain", sources=keychain_src, extra_link_args=keychain_link_args)
+        return Extension("miro.plat.keychain", sources=keychain_src, extra_link_args=keychain_link_args)
     
     def get_qtcomp_ext(self):
         qtcomp_src = glob(os.path.join(ROOT_DIR, 'platform', 'osx', 'modules', 'qtcomp.c'))
         qtcomp_link_args = ['-framework', 'CoreFoundation', '-framework', 'Quicktime']
-        return Extension("miro.platform.qtcomp", sources=qtcomp_src, extra_link_args=qtcomp_link_args)
+        return Extension("miro.plat.qtcomp", sources=qtcomp_src, extra_link_args=qtcomp_link_args)
     
-    def get_database_ext(self):
-        database_src = glob(os.path.join(ROOT_DIR, 'portable', 'database.pyx'))
-        return Extension("miro.database", sources=database_src)
+    def get_growl_ext(self):
+        growl_src = glob(os.path.join(ROOT_DIR, 'platform', 'osx', 'modules', '_growl.c'))
+        growl_link_args = ['-framework', 'CoreFoundation']
+        return Extension("miro.plat._growl", sources=growl_src, extra_link_args=growl_link_args)
+    
+    def get_growl_image_ext(self):
+        growl_image_src = glob(os.path.join(ROOT_DIR, 'platform', 'osx', 'modules', '_growlImage.m'))
+        growl_image_link_args = ['-framework', 'Cocoa']
+        return Extension("miro.plat._growlImage", sources=growl_image_src, extra_link_args=growl_image_link_args)
+
+    def get_shading_ext(self):
+        shading_src = glob(os.path.join(ROOT_DIR, 'platform', 'osx', 'modules', 'shading.m'))
+        shading_link_args = ['-framework', 'ApplicationServices']
+        return Extension("miro.plat.shading", sources=shading_src, extra_link_args=shading_link_args)
     
     def get_sorts_ext(self):
         sorts_src = glob(os.path.join(ROOT_DIR, 'portable', 'sorts.pyx'))
@@ -368,11 +376,12 @@ class MiroBuild (py2app):
                 if '.svn' in dirs:
                     dirs.remove('.svn')
                 for file in files:
-                    if file.endswith('.cpp'):
+                    if file.endswith('.cpp') or file.endswith('.c'):
                         yield os.path.join(root,file)
 
         libtorrent_src = list(libtorrent_sources_iterator())
-        libtorrent_src.remove(os.path.join(PORTABLE_DIR, 'libtorrent/src/file_win.cpp'))
+        if os.path.exists(os.path.join(PORTABLE_DIR, 'libtorrent/src/file_win.cpp')):
+            libtorrent_src.remove(os.path.join(PORTABLE_DIR, 'libtorrent/src/file_win.cpp'))
         libtorrent_inc_dirs = [BOOST_INCLUDE_DIR,
                                os.path.join(PORTABLE_DIR, 'libtorrent', 'include'),
                                os.path.join(PORTABLE_DIR, 'libtorrent', 'include', 'libtorrent')]
@@ -380,15 +389,19 @@ class MiroBuild (py2app):
         libtorrent_libs = ['z', 
                            'pthread', 
                            'ssl']
+
         libtorrent_extras = [PYTHON_LIB,
                              BOOST_PYTHON_LIB,
                              BOOST_FILESYSTEM_LIB,
                              BOOST_DATETIME_LIB,
-                             BOOST_THREAD_LIB]
+                             BOOST_THREAD_LIB,
+                             BOOST_SYSTEM_LIB]
+
         libtorrent_compil_args = ["-Wno-missing-braces",
-                                  "-DHAVE_INCLUDE_LIBTORRENT_ASIO____ASIO_HPP=1", 
-                                  "-DHAVE_INCLUDE_LIBTORRENT_ASIO_SSL_STREAM_HPP=1", 
-                                  "-DHAVE_INCLUDE_LIBTORRENT_ASIO_IP_TCP_HPP=1", 
+                                  "-D_FILE_OFFSET_BITS=64",
+                                  "-DHAVE___INCLUDE_LIBTORRENT_ASIO_HPP=1",
+                                  "-DHAVE___INCLUDE_LIBTORRENT_ASIO_SSL_STREAM_HPP=1",
+                                  "-DHAVE___INCLUDE_LIBTORRENT_ASIO_IP_TCP_HPP=1",
                                   "-DHAVE_PTHREAD=1", 
                                   "-DTORRENT_USE_OPENSSL=1", 
                                   "-DHAVE_SSL=1",
@@ -411,13 +424,12 @@ class MiroBuild (py2app):
     def run(self):
         print "Building %s v%s (%s)" % (self.config.get('longAppName'), self.config.get('appVersion'), self.config.get('appRevision'))
         
-        self.setup_templates()
-        self.setup_dynamic_sources()
         self.setup_info_plist()
 
         py2app.run(self)
         
         self.fix_frameworks_alias()
+        self.precompile_site_pyc()
         self.copy_quicktime_components()
         self.copy_portable_resources()
         self.copy_config_file()
@@ -431,15 +443,6 @@ class MiroBuild (py2app):
         if self.make_dmg:
             self.make_disk_image()
     
-    def setup_templates(self):
-        template_compiler.compileAllTemplates(ROOT_DIR)
-    
-    def setup_dynamic_sources(self):
-        daemonTemplate = os.path.join(ROOT_DIR, 'portable/dl_daemon/daemon.py.template')
-        target = os.path.join(ROOT_DIR, 'portable/dl_daemon/daemon.py')
-        data = self.config.get_data()
-        self.fillTemplate(daemonTemplate, target, **data)
-    
     def setup_info_plist(self):
         def updatePListEntry(plist, key, conf, prioritizeTheme=True):
             entry = plist[key]
@@ -453,6 +456,10 @@ class MiroBuild (py2app):
         updatePListEntry(infoPlist, u'CFBundleShortVersionString', self.config)
         updatePListEntry(infoPlist, u'CFBundleVersion', self.config)
         updatePListEntry(infoPlist, u'NSHumanReadableCopyright', self.config)
+
+        infoPlist['CFBundleDevelopmentRegion'] = 'en'
+        infoPlist['CFBundleAllowMixedLocalizations'] = True
+        infoPlist['CFBundleLocalizations'] = self.get_localizations_list()
         
         self.plist = infoPlist
 
@@ -481,7 +488,7 @@ class MiroBuild (py2app):
             print "    (all skipped, already bundled)"
         else:
             os.mkdir(self.prsrcRoot)
-            for resource in ('css', 'images', 'html', 'searchengines', 'dtvapi.js', 'statictabs.xml'):
+            for resource in ('searchengines', 'images'):
                 src = os.path.join(ROOT_DIR, 'resources', resource)
                 rsrcName = os.path.basename(src)
                 if os.path.isdir(src):
@@ -492,21 +499,20 @@ class MiroBuild (py2app):
                     copy = shutil.copy
                 print "    %s" % dest
                 copy(src, dest)
-            os.mkdir(os.path.join(self.prsrcRoot, 'templates'))
-            if os.environ.has_key('DEMOCRACY_RECOMPILE_TEMPLATES'):
-                template_items = glob(os.path.join(ROOT_DIR, 'resources', 'templates', '*'))
-            else:
-                template_items = glob(os.path.join(ROOT_DIR, 'resources', 'templates', '*.js'))
-            for template_item in template_items:
-                dest = os.path.join(self.prsrcRoot, 'templates', os.path.basename(template_item))
-                print "    %s" % dest
-                if not os.path.isdir(template_item):
-                    copy(template_item, dest)
-                
+
     def copy_config_file(self):
         print "Copying config file to application bundle"
         shutil.move(self.appConfigPath, os.path.join(self.prsrcRoot, 'app.config'))
 
+    def get_localizations_list(self):
+        localeDir = os.path.join(ROOT_DIR, 'resources', 'locale')
+        entries = os.listdir(localeDir)
+        localizations = list()
+        for e in entries:
+            if e.endswith('.po'):
+                localizations.append(os.path.splitext(e)[0])
+        return localizations
+
     def copy_localization_files(self):
         # Copy the gettext MO files in a 'locale' folder inside the
         # application bundle resources folder. Doing this manually at
@@ -515,7 +521,7 @@ class MiroBuild (py2app):
         # 'locale' folder.
         print "Copying gettext MO files to application bundle"
 
-        localeDir = os.path.join(ROOT_DIR, 'resources/locale')
+        localeDir = os.path.join(ROOT_DIR, 'resources', 'locale')
         lclDir = os.path.join(self.rsrcRoot, 'locale')
         if self.force_update and os.path.exists(lclDir):
             shutil.rmtree(lclDir, True);
@@ -530,6 +536,11 @@ class MiroBuild (py2app):
                 shutil.copy2(source, dest)
                 print "    %s" % dest
     
+    def precompile_site_pyc(self):
+        print "Pre-compiling site.py"
+        import py_compile
+        py_compile.compile(os.path.join(self.rsrcRoot, 'lib', 'python%s' % PYTHON_VERSION, 'site.py'))
+    
     def copy_theme_files(self):
         # Copy theme files to the application bundle
         print "Copying theme file"
@@ -544,10 +555,16 @@ class MiroBuild (py2app):
             print "    (all skipped, already bundled)"
         else:
             os.makedirs(targetDir)
-            for f in glob(os.path.join(sourceDir, "*")):
-                dest = os.path.join(targetDir, os.path.basename(f))
-                print "    %s" % dest
-                shutil.copy(f, dest)
+            def copyDirectory(d):
+                for f in glob(os.path.join(d, "*")):
+                    dest = os.path.join(targetDir, f[len(sourceDir)+1:])
+                    print "    %s" % dest
+                    if os.path.isfile(f):
+                        shutil.copy(f, dest)
+                    elif os.path.isdir(f):
+                        os.makedirs(dest)
+                        copyDirectory(f)
+            copyDirectory(sourceDir)
     
     def clean_up_incomplete_lproj(self):
         print "Wiping out incomplete lproj folders"
@@ -676,37 +693,25 @@ class MiroClean (Command):
                 shutil.rmtree(appl)
         except:
             pass
-        print "Removing old compiled templates..."
-        try:
-            template_dir = os.path.join("..","..","portable", "frontends",
-                    "html", "compiled_templates")
-            for filename in os.listdir(template_dir):
-                filename = os.path.join(template_dir, filename)
-                if not filename.startswith(".svn"):
-                    filename = os.path.join("..","..","portable","compiled_templates",filename)
-                    if os.path.isdir(filename):
-                        shutil.rmtree(filename)
-                    else:
-                        os.remove(filename)
-        except:
-            pass
 
-# =============================================================================
-# This is weird, if we do this from the MiroBuild command we eventually get an
-# error in the macholib module... So let's just do it here.
-# =============================================================================
+if __name__ == "__main__":
+    # =========================================================================
+    # This is weird, if we do this from the MiroBuild command we eventually get
+    # an error in the macholib module... So let's just do it here.
+    # =========================================================================
 
-print 'Extracting frameworks to build directory...'
-frameworks_path = os.path.join(ROOT_DIR, 'platform/osx/build/frameworks')
-extract_binaries('frameworks', frameworks_path, True)
-frameworks = glob(os.path.join(frameworks_path, '*.framework'))
+    print 'Extracting frameworks to build directory...'
+    frameworks_path = os.path.join(ROOT_DIR, 'platform/osx/build/frameworks')
+    extract_binaries('frameworks', frameworks_path, True)
+    frameworks = glob(os.path.join(frameworks_path, '*.framework'))
 
-# =============================================================================
-# Launch the setup process...
-# =============================================================================
+    # =========================================================================
+    # Launch the setup process...
+    # =========================================================================
 
-from Pyrex.Distutils import build_ext
-from distutils.core import setup
+    from Pyrex.Distutils import build_ext
+    from distutils.core import setup
 
-setup( cmdclass = {'build_ext': build_ext, 'clean': MiroClean, 'py2app': MiroBuild}, 
-       options  = {'py2app':{'frameworks': frameworks}} )
+    setup( cmdclass = {'build_ext': build_ext, 'clean': MiroClean,
+                       'py2app': MiroBuild},
+           options  = {'py2app':{'frameworks': frameworks}} )
