@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2008 Participatory Culture Foundation
+# Copyright (C) 2005-2009 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,34 +28,31 @@
 
 from base64 import b64encode
 from miro.gtcache import gettext as _
-from threading import RLock
 import os
-import re
-import shutil
 
-from miro.database import DDBObject, defaultDatabase
+from miro.database import DDBObject, ObjectNotFoundError
 from miro.dl_daemon import daemon, command
 from miro.download_utils import nextFreeFilename, getFileURLPath, filterDirectoryName
-from miro.util import getTorrentInfoHash, returnsUnicode, checkU, returnsFilename, unicodify, checkF, stringify, toUni
-from miro import app
+from miro.util import get_torrent_info_hash, returnsUnicode, checkU, returnsFilename, unicodify, checkF, toUni
+# from miro import app
 from miro import config
+from miro import dialogs
 from miro import httpclient
-from miro import indexes
 from miro import prefs
 import random
-from miro import views
-from miro.platform.utils import samefile, FilenameType, unicodeToFilename
+from miro.plat.utils import samefile, FilenameType, unicodeToFilename
 from miro import flashscraper
 import logging
-import traceback
 from miro import fileutil
 
+daemon_starter = None
+
 # a hash of download ids that the server knows about.
 _downloads = {}
 
 # Returns an HTTP auth object corresponding to the given host, path or
 # None if it doesn't exist
-def findHTTPAuth(host,path,realm = None,scheme = None):
+def find_http_auth(host, path, realm=None, scheme=None):
     checkU(host)
     checkU(path)
     if realm:
@@ -63,99 +60,122 @@ def findHTTPAuth(host,path,realm = None,scheme = None):
     if scheme:
         checkU(scheme)
     #print "Trying to find HTTPAuth with host %s, path %s, realm %s, and scheme %s" %(host,path,realm,scheme)
-    defaultDatabase.confirmDBThread()
-    for obj in views.httpauths:
+    for obj in HTTPAuthPassword.make_view():
         if (obj.host == host and path.startswith(obj.path) and
-            (realm is None or obj.realm == realm) and
-            (scheme is None or obj.authScheme == scheme)):
+                (realm is None or obj.realm == realm) and
+                (scheme is None or obj.authScheme == scheme)):
             return obj
     return None
 
 
 class HTTPAuthPassword(DDBObject):
-    def __init__(self,username,password,host, realm, path, authScheme=u"Basic"):
+    def setup_new(self, username, password, host, realm, path, authScheme=u"Basic"):
         checkU(username)
         checkU(password)
         checkU(host)
         checkU(realm)
         checkU(path)
         checkU(authScheme)
-        oldAuth = findHTTPAuth(host,path,realm,authScheme)
+        oldAuth = find_http_auth(host, path, realm, authScheme)
         while not oldAuth is None:
             oldAuth.remove()
-            oldAuth = findHTTPAuth(host,path,realm,authScheme)
+            oldAuth = find_http_auth(host, path, realm, authScheme)
         self.username = username
         self.password = password
         self.host = host
         self.realm = realm
         self.path = os.path.dirname(path)
         self.authScheme = authScheme
-        DDBObject.__init__(self)
 
-    def getAuthToken(self):
+    def get_auth_token(self):
         authString = u':'
-        self.confirmDBThread()
+        self.confirm_db_thread()
         authString = self.username+u':'+self.password
         return b64encode(authString)
 
-    def getAuthScheme(self):
-        self.confirmDBThread()
+    def get_auth_scheme(self):
+        self.confirm_db_thread()
         return self.authScheme
 
 totalUpRate = 0
 totalDownRate = 0
 
-def _getDownloader (dlid):
-    return views.remoteDownloads.getItemWithIndex(indexes.downloadsByDLID, dlid)
+def get_downloader_by_dlid(dlid):
+    try:
+        return RemoteDownloader.get_by_dlid(dlid)
+    except ObjectNotFoundError:
+        return None
 
 @returnsUnicode
-def generateDownloadID():
-    dlid = u"download%08d" % random.randint(0,99999999)
-    while _getDownloader (dlid=dlid):
-        dlid = u"download%08d" % random.randint(0,99999999)
+def generate_dlid():
+    dlid = u"download%08d" % random.randint(0, 99999999)
+    while get_downloader_by_dlid(dlid=dlid):
+        dlid = u"download%08d" % random.randint(0, 99999999)
     return dlid
 
 class RemoteDownloader(DDBObject):
     """Download a file using the downloader daemon."""
-
-    def __init__(self, url, item, contentType = None, channelName = None):
+    def setup_new(self, url, item, contentType=None, channelName=None):
         checkU(url)
         if contentType:
             checkU(contentType)
         self.origURL = self.url = url
-        self.itemList = [item]
-        self.dlid = generateDownloadID()
+        self.itemList = []
+        self.child_deleted = False
+        self.main_item_id = None
+        self.dlid = generate_dlid()
         self.status = {}
+        self.state = u'downloading'
         if contentType is None:
             # HACK:  Some servers report the wrong content-type for torrent
             # files.  We try to work around that by assuming if the enclosure
             # states that something is a torrent, it's a torrent.
             # Thanks to j@v2v.cc
-            enclosureContentType = item.getFirstVideoEnclosureType()
-            if enclosureContentType == u'application/x-bittorrent':
-                contentType = enclosureContentType
+            if item.enclosure_type == u'application/x-bittorrent':
+                contentType = item.enclosure_type
         self.contentType = u""
         self.deleteFiles = True
         self.channelName = channelName
         self.manualUpload = False
-        DDBObject.__init__(self)
         if contentType is None:
             self.contentType = u""
         else:
             self.contentType = contentType
 
         if self.contentType == u'':
-            self.getContentType()
+            self.get_content_type()
         else:
-            self.runDownloader()
+            self.run_downloader()
 
-    def signalChange (self, needsSave=True, needsSignalItem=True):
+    @classmethod
+    def finished_view(cls):
+        return cls.make_view("state in ('finished', 'uploading', "
+                "'uploading-paused')")
+
+    @classmethod
+    def auto_uploader_view(cls):
+        return cls.make_view("state == 'uploading' AND NOT manualUpload")
+
+    @classmethod
+    def get_by_dlid(cls, dlid):
+        return cls.make_view('dlid=?', (dlid,)).get_singleton()
+
+    @classmethod
+    def get_by_url(cls, url):
+        return cls.make_view('origURL=?', (url,)).get_singleton()
+
+    @classmethod
+    def orphaned_view(cls):
+        """Downloaders with no items associated with them."""
+        return cls.make_view('id NOT IN (SELECT downloader_id from item)')
+
+    def signal_change(self, needsSave=True, needsSignalItem=True):
+        DDBObject.signal_change(self, needsSave=needsSave)
         if needsSignalItem:
             for item in self.itemList:
-                item.signalChange(needsSave=False)
-        DDBObject.signalChange (self, needsSave=needsSave)
+                item.signal_change(needsSave=False)
 
-    def onContentType (self, info):
+    def on_content_type(self, info):
         if not self.idExists():
             return
 
@@ -164,57 +184,60 @@ class RemoteDownloader(DDBObject):
             self.contentType = None
             try:
                 self.contentType = info['content-type'].decode('ascii','replace')
+            except (SystemExit, KeyboardInterrupt):
+                raise
             except:
                 self.contentType = None
-            self.runDownloader()
+            self.run_downloader()
         else:
             error = httpclient.UnexpectedStatusCode(info['status'])
-            self.onContentTypeError(error)
+            self.on_content_type_error(error)
 
-    def onContentTypeError (self, error):
+    def on_content_type_error(self, error):
         if not self.idExists():
             return
 
         self.status['state'] = u"failed"
         self.status['shortReasonFailed'] = error.getFriendlyDescription()
         self.status['reasonFailed'] = error.getLongDescription()
-        self.signalChange()
+        self.signal_change()
 
-    def getContentType(self):
-        httpclient.grabHeaders(self.url, self.onContentType, self.onContentTypeError)
+    def get_content_type(self):
+        httpclient.grabHeaders(self.url,
+                               self.on_content_type, self.on_content_type_error)
  
     @classmethod
-    def initializeDaemon(cls):
+    def initialize_daemon(cls):
         RemoteDownloader.dldaemon = daemon.ControllerDaemon()
 
-    def _getRates(self):
-        state = self.getState()
+    def _get_rates(self):
+        state = self.get_state()
         if state == u'downloading':
             return (self.status.get('rate', 0), self.status.get('upRate', 0))
         if state == u'uploading':
             return (0, self.status.get('upRate', 0))
         return (0, 0)
 
-    def beforeChangingStatus(self):
+    def before_changing_status(self):
         global totalDownRate
         global totalUpRate
-        rates = self._getRates()
+        rates = self._get_rates()
         totalDownRate -= rates[0]
         totalUpRate -= rates[1]
 
-    def afterChangingStatus(self):
+    def after_changing_status(self):
         global totalDownRate
         global totalUpRate
-        rates = self._getRates()
+        rates = self._get_rates()
         totalDownRate += rates[0]
         totalUpRate += rates[1]
 
     @classmethod
-    def updateStatus(cls, data):
+    def update_status(cls, data):
         for field in data:
-            if field not in ['filename','shortFilename','channelName','metainfo','fastResumeData']:
+            if field not in ['filename', 'shortFilename', 'channelName', 'metainfo', 'fastResumeData']:
                 data[field] = unicodify(data[field])
-        self = _getDownloader (dlid=data['dlid'])
+        self = get_downloader_by_dlid(dlid=data['dlid'])
         # print data
         if self is not None:
             try:
@@ -226,7 +249,8 @@ class RemoteDownloader(DDBObject):
                 print "WARNING exception when comparing status: %s" % e
 
             wasFinished = self.isFinished()
-            self.beforeChangingStatus()
+            old_filename = self.get_filename()
+            self.before_changing_status()
 
             # FIXME: how do we get all of the possible bit torrent
             # activity strings into gettext? --NN
@@ -234,31 +258,38 @@ class RemoteDownloader(DDBObject):
                 data['activity'] = _(data['activity'])
 
             self.status = data
+            self._recalc_state()
 
             # Store the time the download finished
             finished = self.isFinished() and not wasFinished
-            self.afterChangingStatus()
+            file_migrated = (self.isFinished() and
+                    self.get_filename() != old_filename)
+            needsSignalItem = not (finished or file_migrated)
+            self.after_changing_status()
 
-            if self.getState() == u'uploading' and not self.manualUpload and self.getUploadRatio() > 1.5:
+            if self.get_state() == u'uploading' and not self.manualUpload and self.getUploadRatio() > 1.5:
                 self.stopUpload()
 
-            self.signalChange(needsSignalItem=not finished)
+            self.signal_change(needsSignalItem = needsSignalItem)
             if finished:
                 for item in self.itemList:
-                    item.onDownloadFinished()
+                    item.on_download_finished()
+            elif file_migrated:
+                self._file_migrated(old_filename)
 
-    ##
-    # This is the actual download thread.
-    def runDownloader(self):
-        flashscraper.tryScrapingURL(self.url, self._runDownloader)
+    def run_downloader(self):
+        """This is the actual download thread.
+        """
+        flashscraper.try_scraping_url(self.url, self._run_downloader)
 
-    def _runDownloader(self, url, contentType = None):
+    def _run_downloader(self, url, contentType = None):
         if not self.idExists():
             return # we got deleted while we were doing the flash scraping
         if contentType is not None:
             self.contentType = contentType
         if url is not None:
             self.url = url
+            logging.debug("downloading url %s" % self.url)
             c = command.StartNewDownloadCommand(RemoteDownloader.dldaemon,
                                                 self.url, self.dlid, self.contentType, self.channelName)
             c.send()
@@ -267,26 +298,24 @@ class RemoteDownloader(DDBObject):
             self.status["state"] = u'failed'
             self.status["shortReasonFailed"] = _('File not found')
             self.status["reasonFailed"] = _('Flash URL Scraping Error')
-        self.signalChange()
+        self.signal_change()
 
-    ##
-    # Pauses the download.
-    def pause(self, block=False):
+    def pause(self):
+        """Pauses the download."""
         if _downloads.has_key(self.dlid):
-            c = command.PauseDownloadCommand(RemoteDownloader.dldaemon,
-                                             self.dlid)
+            c = command.PauseDownloadCommand(RemoteDownloader.dldaemon, self.dlid)
             c.send()
         else:
-            self.beforeChangingStatus()
+            self.before_changing_status()
             self.status["state"] = u"paused"
-            self.afterChangingStatus()
-            self.signalChange()
+            self.after_changing_status()
+            self.signal_change()
 
-    ##
-    # Stops the download and removes the partially downloaded
-    # file.
     def stop(self, delete):
-        if ((self.getState() in [u'downloading',u'uploading', u'paused'])):
+        """Stops the download and removes the partially downloaded
+        file.
+        """
+        if self.get_state() in [u'downloading', u'uploading', u'paused']:
             if _downloads.has_key(self.dlid):
                 c = command.StopDownloadCommand(RemoteDownloader.dldaemon,
                                                 self.dlid, delete)
@@ -296,7 +325,7 @@ class RemoteDownloader(DDBObject):
             if delete:
                 self.delete()
             self.status["state"] = u"stopped"
-            self.signalChange()
+            self.signal_change()
 
     def delete(self):
         try:
@@ -305,39 +334,36 @@ class RemoteDownloader(DDBObject):
             return
         try:
             fileutil.delete(filename)
-        except:
-            logging.warn("Error deleting downloaded file: %s\n%s" % 
-                    (toUni(filename), traceback.format_exc()))
+        except OSError:
+            logging.exception("Error deleting downloaded file: %s", toUni(filename))
 
-        parent = os.path.join(filename, os.path.pardir)
+        parent = os.path.join(fileutil.expand_filename(filename), os.path.pardir)
         parent = os.path.normpath(parent)
-        moviesDir = config.get(prefs.MOVIES_DIRECTORY)
+        moviesDir = fileutil.expand_filename(config.get(prefs.MOVIES_DIRECTORY))
         if (os.path.exists(parent) and os.path.exists(moviesDir) and
-            not samefile(parent, moviesDir) and
-            len(os.listdir(parent)) == 0):
+                not samefile(parent, moviesDir) and
+                len(os.listdir(parent)) == 0):
             try:
                 os.rmdir(parent)
-            except:
-                logging.warn("Error deleting empty download directory: %s\n%s" %
-                        (toUni(parent), traceback.format_exc()))
+            except OSError:
+                logging.exception("Error deleting empty download directory: %s", toUni(parent))
 
-    ##
-    # Continues a paused, stopped, or failed download thread
     def start(self):
-        if self.getState() == u'failed':
+        """Continues a paused, stopped, or failed download thread
+        """
+        if self.get_state() == u'failed':
             if _downloads.has_key (self.dlid):
                 del _downloads[self.dlid]
-            self.dlid = generateDownloadID()
-            views.remoteDownloads.recomputeIndex(indexes.downloadsByDLID)
-            self.beforeChangingStatus()
+            self.dlid = generate_dlid()
+            self.before_changing_status()
             self.status = {}
-            self.afterChangingStatus()
+            self.after_changing_status()
             if self.contentType == u"":
-                self.getContentType()
+                self.get_content_type()
             else:
-                self.runDownloader()
-            self.signalChange()
-        elif self.getState() in (u'stopped', u'paused', u'offline'):
+                self.run_downloader()
+            self.signal_change()
+        elif self.get_state() in (u'stopped', u'paused', u'offline'):
             if _downloads.has_key(self.dlid):
                 c = command.StartDownloadCommand(RemoteDownloader.dldaemon,
                                                  self.dlid)
@@ -345,7 +371,7 @@ class RemoteDownloader(DDBObject):
             else:
                 self.status['state'] = u'downloading'
                 self.restart()
-                self.signalChange()
+                self.signal_change()
 
     def migrate(self, directory):
         if _downloads.has_key(self.dlid):
@@ -368,13 +394,13 @@ URL was %s""" % self.url
 WARNING: can't migrate download because we don't have a filename!
 URL was %s""" % self.url
                 return
-            if os.path.exists(filename):
+            if fileutil.exists(filename):
                 if 'channelName' in self.status and self.status['channelName'] is not None:
                     channelName = filterDirectoryName(self.status['channelName'])
                     directory = os.path.join (directory, channelName)
                 try:
-                    os.makedirs(directory)
-                except:
+                    fileutil.makedirs(directory)
+                except OSError:
                     pass
                 newfilename = os.path.join(directory, shortFilename)
                 if newfilename == filename:
@@ -382,146 +408,164 @@ URL was %s""" % self.url
                 newfilename = nextFreeFilename(newfilename)
                 def callback():
                     self.status['filename'] = newfilename
-                    self.signalChange()
+                    self.signal_change(needsSignalItem=False)
+                    self._file_migrated(filename)
                 fileutil.migrate_file(filename, newfilename, callback)
         for i in self.itemList:
-            i.migrateChildren(directory)
+            i.migrate_children(directory)
 
-    def setDeleteFiles(self, deleteFiles):
+    def _file_migrated(self, old_filename):
+        for item in self.itemList:
+            item.on_downloader_migrated(old_filename, self.get_filename())
+
+    def set_delete_files(self, deleteFiles):
         self.deleteFiles = deleteFiles
 
-    def setChannelName(self, channelName):
+    def set_channel_name(self, channelName):
         if self.channelName is None:
             if channelName:
                 checkF(channelName)
             self.channelName = channelName
 
-    ##
-    # Removes downloader from the database and deletes the file.
     def remove(self):
+        """Removes downloader from the database and deletes the file.
+        """
         global totalDownRate
         global totalUpRate
-        rates = self._getRates()
+        rates = self._get_rates()
         totalDownRate -= rates[0]
         totalUpRate -= rates[1]
         self.stop(self.deleteFiles)
         DDBObject.remove(self)
 
-    def getType(self):
+    def get_type(self):
         """Get the type of download.  Will return either "http" or
         "bittorrent".
         """
-        self.confirmDBThread()
+        self.confirm_db_thread()
         if self.contentType == u'application/x-bittorrent':
             return u"bittorrent"
         else:
             return u"http"
 
-    ##
-    # In case multiple downloaders are getting the same file, we can support
-    # multiple items
-    def addItem(self,item):
+    def addItem(self, item):
+        """In case multiple downloaders are getting the same file, we can support
+        multiple items
+        """
         if item not in self.itemList:
             self.itemList.append(item)
+            if self.main_item_id is None:
+                self.main_item_id = item.id
+                self.signal_change()
 
     def removeItem(self, item):
         self.itemList.remove(item)
         if len (self.itemList) == 0:
             self.remove()
+        elif item.id == self.main_item_id:
+            self.main_item_id = self.itemList[0].id
+            self.signal_change()
 
     def getRate(self):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get('rate', 0)
 
     def getETA(self):
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status.get('eta', 0)
 
     @returnsUnicode
-    def getStartupActivity(self):
-        self.confirmDBThread()
+    def get_startup_activity(self):
+        self.confirm_db_thread()
         activity = self.status.get('activity')
         if activity is None:
             return _("starting up")
         else:
             return activity
 
-    ##
-    # Returns the reason for the failure of this download
-    # This should only be called when the download is in the failed state
     @returnsUnicode
     def getReasonFailed(self):
-        if not self.getState() == u'failed':
+        """Returns the reason for the failure of this download
+        This should only be called when the download is in the failed state
+        """
+        if not self.get_state() == u'failed':
             msg = u"getReasonFailed() called on a non-failed downloader"
             raise ValueError(msg)
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status['reasonFailed']
 
     @returnsUnicode
     def getShortReasonFailed(self):
-        if not self.getState() == u'failed':
+        if not self.get_state() == u'failed':
             msg = u"getShortReasonFailed() called on a non-failed downloader"
             raise ValueError(msg)
-        self.confirmDBThread()
+        self.confirm_db_thread()
         return self.status['shortReasonFailed']
-    ##
-    # Returns the URL we're downloading
+
     @returnsUnicode
-    def getURL(self):
-        self.confirmDBThread()
+    def get_url(self):
+        """Returns the URL we're downloading
+        """
+        self.confirm_db_thread()
         return self.url
 
-    ##    
-    # Returns the state of the download: downloading, paused, stopped,
-    # failed, or finished
     @returnsUnicode    
-    def getState(self):
-        self.confirmDBThread()
-        return self.status.get('state', u'downloading')
+    def get_state(self):
+        """Returns the state of the download: downloading, paused, stopped,
+        failed, or finished
+        """
+        self.confirm_db_thread()
+        return self.state
 
     def isFinished(self):
-        return self.getState() in (u'finished', u'uploading')
+        return self.get_state() in (u'finished', u'uploading', u'uploading-paused')
 
-    ##
-    # Returns the total size of the download in bytes
     def getTotalSize(self):
-        self.confirmDBThread()
+        """Returns the total size of the download in bytes
+        """
+        self.confirm_db_thread()
         return self.status.get(u'totalSize', -1)
 
-    ##
-    # Returns the current amount downloaded in bytes
-    def getCurrentSize(self):
-        self.confirmDBThread()
+    def get_current_size(self):
+        """Returns the current amount downloaded in bytes
+        """
+        self.confirm_db_thread()
         return self.status.get(u'currentSize', 0)
 
-    ##
-    # Returns the filename that we're downloading to. Should not be
-    # called until state is "finished."
     @returnsFilename
-    def getFilename(self):
-        self.confirmDBThread()
+    def get_filename(self):
+        """Returns the filename that we're downloading to. Should not be
+        called until state is "finished."
+        """
+        self.confirm_db_thread()
         return self.status.get('filename', FilenameType(''))
 
-    def onRestore(self):
+    def setup_restored(self):
         self.deleteFiles = True
         self.itemList = []
         if self.dlid == 'noid':
             # this won't happen nowadays, but it can for old databases
-            self.dlid = generateDownloadID()
+            self.dlid = generate_dlid()
         self.status['rate'] = 0
         self.status['upRate'] = 0
         self.status['eta'] = 0
 
+    def on_signal_change(self):
+        self._recalc_state()
+
+    def _recalc_state(self):
+        self.state = self.status.get('state', u'downloading')
+
     def getUploadRatio(self):
-        size = self.getCurrentSize()
+        size = self.get_current_size()
         if size == 0:
             return 0
         return self.status.get('uploaded', 0) / size
     
     def restartIfNeeded(self):
-        if self.getState() in (u'downloading',u'offline'):
+        if self.get_state() in (u'downloading', u'offline'):
             self.restart()
-        if self.getState() in (u'uploading'):
+        if self.get_state() in (u'uploading'):
             if self.manualUpload or self.getUploadRatio() < 1.5:
                 self.restart()
             else:
@@ -530,9 +574,9 @@ URL was %s""" % self.url
     def restart(self):
         if len(self.status) == 0 or self.status.get('dlerType') is None:
             if self.contentType == u"":
-                self.getContentType()
+                self.get_content_type()
             else:
-                self.runDownloader()
+                self.run_downloader()
         else:
             _downloads[self.dlid] = self
             c = command.RestoreDownloaderCommand(RemoteDownloader.dldaemon, 
@@ -540,7 +584,18 @@ URL was %s""" % self.url
             c.send()
 
     def startUpload(self):
-        if self.getState() != u'finished' or self.getType() != u'bittorrent':
+        if self.get_type() != u'bittorrent':
+            logging.warn("called startUpload for non-bittorrent downloader")
+            return
+        if self.child_deleted:
+            title = "Can't Resume Seeding"
+            msg = ("Seeding cannot resume because part of this torrent "
+                    "has been deleted.")
+            dialogs.MessageBoxDialog(title, msg).run()
+            return
+        if self.get_state() not in (u'finished', u'uploading-paused'):
+            logging.warn("called startUpload when downloader state "
+                        "is: %s", self.get_state())
             return
         self.manualUpload = True
         if _downloads.has_key(self.dlid):
@@ -548,74 +603,119 @@ URL was %s""" % self.url
                                              self.dlid)
             c.send()
         else:
-            self.beforeChangingStatus()
+            self.before_changing_status()
             self.status['state'] = u'uploading'
-            self.afterChangingStatus()
+            self.after_changing_status()
             self.restart()
-            self.signalChange()
+            self.signal_change()
 
     def stopUpload(self):
-        if self.getState() != u"uploading":
-            return
+        """
+        Stop uploading/seeding and set status as "finished".
+        """
         if _downloads.has_key(self.dlid):
             c = command.StopUploadCommand(RemoteDownloader.dldaemon,
                                           self.dlid)
             c.send()
             del _downloads[self.dlid]
-        self.beforeChangingStatus()
+        self.before_changing_status()
         self.status["state"] = u"finished"
-        self.afterChangingStatus()
-        self.signalChange()
+        self.after_changing_status()
+        self.signal_change()
 
-def cleanupIncompleteDownloads():
+    def pauseUpload(self):
+        """
+        Stop uploading/seeding and set status as "uploading-paused".
+        """
+        if _downloads.has_key(self.dlid):
+            c = command.PauseUploadCommand(RemoteDownloader.dldaemon,
+                                           self.dlid)
+            c.send()
+            del _downloads[self.dlid]
+        self.before_changing_status()
+        self.status["state"] = u"uploading-paused"
+        self.after_changing_status()
+        self.signal_change()
+
+
+def cleanup_incomplete_downloads():
     downloadDir = os.path.join(config.get(prefs.MOVIES_DIRECTORY),
-            'Incomplete Downloads')
-    if not os.path.exists(downloadDir):
+                                          'Incomplete Downloads')
+    if not fileutil.exists(downloadDir):
         return
 
     filesInUse = set()
-    views.remoteDownloads.confirmDBThread()
-    for downloader in views.remoteDownloads:
-        if downloader.getState() in ('downloading', 'paused', 'offline'):
-            filename = downloader.getFilename()
+    for downloader in RemoteDownloader.make_view():
+        if downloader.get_state() in ('downloading', 'paused',
+                                     'offline', 'uploading', 'finished',
+                                     'uploading-paused'):
+            filename = downloader.get_filename()
             if len(filename) > 0:
-                if not os.path.isabs(filename):
+                if not fileutil.isabs(filename):
                     filename = os.path.join(downloadDir, filename)
                 filesInUse.add(filename)
 
-    for f in os.listdir(downloadDir):
+    for f in fileutil.listdir(downloadDir):
         f = os.path.join(downloadDir, f)
         if f not in filesInUse:
             try:
-                if os.path.isfile(f):
-                    os.remove (f)
-                elif os.path.isdir(f):
-                    shutil.rmtree (f)
-            except:
+                if fileutil.isfile(f):
+                    fileutil.remove (f)
+                elif fileutil.isdir(f):
+                    fileutil.rmtree (f)
+            except OSError:
                 # FIXME - maybe a permissions error?
                 pass
 
-def restartDownloads():
-    views.remoteDownloads.confirmDBThread()
-    for downloader in views.remoteDownloads:
-        downloader.restartIfNeeded()
-
-def killUploaders(*args):
+def kill_uploaders(*args):
     torrent_limit = config.get(prefs.UPSTREAM_TORRENT_LIMIT)
-    while (views.autoUploads.len() > torrent_limit):
-        views.autoUploads[0].stopUpload()
+    auto_uploads = list(RemoteDownloader.auto_uploader_view())
+    for dler in auto_uploads[torrent_limit:]:
+        dler.stopUpload()
 
-def configChangeUploaders(key, value):
+def config_change_uploaders(key, value):
     if key == prefs.UPSTREAM_TORRENT_LIMIT.key:
-        killUploaders()
+        kill_uploaders()
 
-def limitUploaders():
-    views.autoUploads.addAddCallback(killUploaders)
-    config.addChangeCallback(configChangeUploaders)
-    killUploaders()
-        
+def limit_uploaders():
+    tracker = RemoteDownloader.auto_uploader_view().make_tracker()
+    tracker.connect('added', kill_uploaders)
+    config.add_change_callback(config_change_uploaders)
+    kill_uploaders()
 
-def startupDownloader():
+class DownloadDaemonStarter(object):
+    def __init__(self):
+        RemoteDownloader.initialize_daemon()
+        self.downloads_at_startup = list(RemoteDownloader.make_view())
+        self.started = False
+
+    def startup(self):
+        cleanup_incomplete_downloads()
+        RemoteDownloader.dldaemon.start_downloader_daemon()
+        limit_uploaders()
+        self.restart_downloads()
+        self.started = True
+
+    def restart_downloads(self):
+        for downloader in self.downloads_at_startup:
+            downloader.restartIfNeeded()
+
+    def shutdown(self, callback):
+        if not self.started:
+            callback()
+        else:
+            RemoteDownloader.dldaemon.shutdown_downloader_daemon(callback=callback)
+
+def init_controller():
+    """Intializes the download daemon controller.
+
+    This doesn't actually start up the downloader daemon, that's done in
+    startup_downloader.  Commands will be queued until then.
+    """
+    global daemon_starter
+    daemon_starter = DownloadDaemonStarter()
+
+def startup_downloader():
     """Initialize the downloaders.
 
     This method currently does 2 things.  It deletes any stale files self in
@@ -623,43 +723,45 @@ def startupDownloader():
     from the database.  It must be called before any RemoteDownloader objects
     get created.
     """
+    daemon_starter.startup()
 
-    cleanupIncompleteDownloads()
-    RemoteDownloader.initializeDaemon()
-    limitUploaders()
-    restartDownloads()
-
-def shutdownDownloader(callback = None):
-    if hasattr(RemoteDownloader, 'dldaemon') and RemoteDownloader.dldaemon is not None:
-        RemoteDownloader.dldaemon.shutdownDownloaderDaemon(callback=callback)
-    else:
+def shutdown_downloader(callback=None):
+    if daemon_starter:
+        daemon_starter.shutdown(callback)
+    elif callback:
         callback()
 
-def lookupDownloader(url):
-    return views.remoteDownloads.getItemWithIndex(indexes.downloadsByURL, url)
+def lookup_downloader(url):
+    try:
+        return RemoteDownloader.get_by_url(url)
+    except ObjectNotFoundError:
+        return None
 
-def getExistingDownloaderByURL(url):
-    downloader = lookupDownloader(url)
+def get_existing_downloader_by_url(url):
+    downloader = lookup_downloader(url)
     return downloader
 
-def getExistingDownloader(item):
-    downloader = lookupDownloader(item.getURL())
-    if downloader:
-        downloader.addItem(item)
-    return downloader
+def get_existing_downloader(item):
+    try:
+        return RemoteDownloader.get_by_id(item.downloader_id)
+    except ObjectNotFoundError:
+        return None
 
-def getDownloader(item):
-    existing = getExistingDownloader(item)
+def get_downloader_for_item(item):
+    existing = get_existing_downloader(item)
     if existing:
         return existing
-    url = item.getURL()
-    channelName = unicodeToFilename(item.getChannelTitle(True))
+    url = item.get_url()
+    existing = get_existing_downloader_by_url(url)
+    if existing:
+        return existing
+    channelName = unicodeToFilename(item.get_channel_title(True))
     if not channelName:
         channelName = None
     if url.startswith(u'file://'):
         path = getFileURLPath(url)
         try:
-            getTorrentInfoHash(path)
+            get_torrent_info_hash(path)
         except ValueError:
             raise ValueError("Don't know how to handle %s" % url)
         except IOError:
