@@ -1,5 +1,5 @@
 # Miro - an RSS based video player application
-# Copyright (C) 2005-2008 Participatory Culture Foundation
+# Copyright (C) 2005-2009 Participatory Culture Foundation
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@ reference (http://www.w3.org/Protocols/rfc2616/rfc2616.html).
 import errno
 import logging
 import re
+import platform
 import socket
 import traceback
 from urlparse import urljoin
@@ -52,8 +53,8 @@ from miro.clock import clock
 from miro import httpauth
 from miro import config
 from miro import prefs
-from miro.download_utils import URIPattern, cleanFilename, parseURL, defaultPort, getFileURLPath, filenameFromURL
-from miro.xhtmltools import URLEncodeDict, multipartEncode
+from miro.download_utils import cleanFilename, parseURL, defaultPort, getFileURLPath, filenameFromURL
+from miro.xhtmltools import url_encode_dict, multipart_encode
 from miro import eventloop
 from miro import util
 import sys
@@ -61,14 +62,21 @@ import time
 import urllib
 from miro import signals
 from miro import trapcall
+from miro.plat.resources import get_osname
 
 PIPELINING_ENABLED = True
 SOCKET_READ_TIMEOUT = 60
 SOCKET_INITIAL_READ_TIMEOUT = 30
 SOCKET_CONNECT_TIMEOUT = 15
 
+def user_agent():
+    return "%s/%s (%s; %s)" % (config.get(prefs.SHORT_APP_NAME),
+            config.get(prefs.APP_VERSION),
+            config.get(prefs.PROJECT_URL),
+            get_osname())
+
 class NetworkError(Exception):
-    """Base class for all errors that will be passed to errbacks from getURL
+    """Base class for all errors that will be passed to errbacks from get_url
     and friends.  NetworkErrors can be display in 2 ways:
 
     getFriendlyDescription() -- short, newbie friendly description 
@@ -78,7 +86,7 @@ class NetworkError(Exception):
     def __init__(self, shortDescription, longDescription=None):
         if longDescription is None:
             longDescription = shortDescription
-        self.friendlyDescription = _("Error: %s") % shortDescription
+        self.friendlyDescription = _("Error: %(msg)s", {"msg": shortDescription})
         self.longDescription = longDescription
 
     def getFriendlyDescription(self):
@@ -94,8 +102,8 @@ class NetworkError(Exception):
 class ConnectionError(NetworkError):
     def __init__(self, errorMessage):
         self.friendlyDescription = _("Can't connect")
-        self.longDescription = _("Connection Error: %s") % \
-                               util.stringify(errorMessage, "replace")
+        self.longDescription = _("Connection Error: %(msg)s",
+                                 {"msg": util.unicodify(errorMessage)})
 
 class SSLConnectionError(ConnectionError):
     def __init__(self):
@@ -105,21 +113,30 @@ class SSLConnectionError(ConnectionError):
 class HTTPError(NetworkError):
     def __init__(self, longDescription):
         NetworkError.__init__(self, _("HTTP error"), longDescription)
+
 class BadStatusLine(HTTPError):
     def __init__(self, line):
-        HTTPError.__init__(self, _("Bad Status Line: %s") % line)
+        HTTPError.__init__(self, _("Bad Status Line: %(msg)s",
+                                   {"msg": util.unicodify(line)}))
+
 class BadHeaderLine(HTTPError):
     def __init__(self, line):
-        HTTPError.__init__(self, _("Bad Header Line: %s") % line)
+        HTTPError.__init__(self, _("Bad Header Line: %(msg)s",
+                                   {"msg": util.unicodify(line)}))
+
 class BadChunkSize(HTTPError):
     def __init__(self, line):
-        HTTPError.__init__(self, _("Bad Chunk size: %s") % line)
+        HTTPError.__init__(self, _("Bad Chunk size: %(msg)s",
+                                   {"msg": util.unicodify(line)}))
+
 class CRLFExpected(HTTPError):
     def __init__(self, crlf):
-        HTTPError.__init__(self, _("Expected CRLF got: %r") % crlf)
+        HTTPError.__init__(self, _("Expected CRLF got: %(character)r",
+                                   {"character": crlf}))
+
 class ServerClosedConnection(HTTPError):
     def __init__(self, host):
-        HTTPError.__init__(self, _('%s closed connection') % host)
+        HTTPError.__init__(self, _('%(host)s closed connection', {"host": host}))
 
 class UnexpectedStatusCode(HTTPError):
     def __init__(self, code):
@@ -127,7 +144,8 @@ class UnexpectedStatusCode(HTTPError):
             self.friendlyDescription = _("File not found")
             self.longDescription = _("Got 404 status code")
         else:
-            HTTPError.__init__(self, _("Bad Status Code: %s") % code)
+            HTTPError.__init__(self, _("Bad Status Code: %(code)s",
+                                       {"code": util.unicodify(code)}))
 
 class AuthorizationFailed(NetworkError):
     def __init__(self):
@@ -142,32 +160,32 @@ class PipelinedRequestNeverStarted(NetworkError):
 class ConnectionTimeout(NetworkError):
     def __init__(self, host):
         NetworkError.__init__(self, _('Timeout'),
-                _('Connection to %s timed out') % host)
+                _('Connection to %(host)s timed out', {"host": host}))
 
 class MalformedURL(NetworkError):
     def __init__(self, url):
         NetworkError.__init__(self, _('Invalid URL'),
-                _('"%s" is not a valid URL') % url)
+                _('"%(url)s" is not a valid URL', {"url": url}))
 
 class FileURLNotFoundError(NetworkError):
     """A file: URL doesn't exist"""
     def __init__(self, path):
         NetworkError.__init__(self, _('File not found'),
-            _('The file: "%s" doesn\'t exist') % path)
+            _('The file "%(path)s" doesn\'t exist', {"path": path}))
 
 class FileURLReadError(NetworkError):
     def __init__(self, path):
         NetworkError.__init__(self, _('Read error'),
-            _('Error while reading from "%s"') % path)
+            _('Error while reading from "%(path)s"', {"path": path}))
 
-def trapCall(object, function, *args, **kwargs):
-    """Convenience function do a trapcall.trapCall, where when is
+def trap_call(object, function, *args, **kwargs):
+    """Convenience function do a trapcall.trap_call, where when is
     'While talking to the network'
     """
-    return trapcall.timeTrapCall("Calling %s on %s" % (function, object), function, *args, **kwargs)
+    return trapcall.time_trap_call("Calling %s on %s" % (function, object), function, *args, **kwargs)
 
 
-DATEINFUTURE = time.mktime( (2030, 7, 12, 12, 0, 0, 4, 193, -1) )
+DATEINFUTURE = time.mktime((2030, 7, 12, 12, 0, 0, 4, 193, -1))
 
 def get_cookie_expiration_date(val):
     """Tries a bunch of possible cookie expiration date formats
@@ -176,7 +194,8 @@ def get_cookie_expiration_date(val):
     fmts = ( '%a, %d %b %Y %H:%M:%S %Z',
              '%a, %d %b %y %H:%M:%S %Z',
              '%a, %d-%b-%Y %H:%M:%S %Z',
-             '%a, %d-%b-%y %H:%M:%S %Z' )
+             '%a, %d-%b-%y %H:%M:%S %Z',
+             '%A, %d-%b-%y %H:%M:%S %Z' )
     
     for fmt in fmts:
         try:
@@ -256,7 +275,7 @@ class NetworkBuffer(object):
 class _Packet(object):
     """A packet of data for the AsyncSocket class
     """
-    def __init__ (self, data, callback = None):
+    def __init__(self, data, callback=None):
         self.data = data
         self.callback = callback
 
@@ -310,7 +329,7 @@ class AsyncSocket(object):
             self.readTimeout.cancel()
             self.readTimeout = None
 
-    def openConnection(self, host, port, callback, errback, disableReadTimeout = None):
+    def openConnection(self, host, port, callback, errback, disableReadTimeout=None):
         """Open a connection.  On success, callback will be called with this
         object.
         """
@@ -321,12 +340,12 @@ class AsyncSocket(object):
         try:
             self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         except socket.error, e:
-            trapCall(self, errback, ConnectionError(e[1]))
+            trap_call(self, errback, ConnectionError(e[1]))
             return
         self.socket.setblocking(0)
         self.connectionErrback = errback
         def handleGetHostByNameException(e):
-            trapCall(self, errback, ConnectionError(e[1]))
+            trap_call(self, errback, ConnectionError(e[1] + " (host: %s)" % host))
         def onAddressLookup(address):
             if self.socket is None:
                 # the connection was closed while we were calling gethostbyname
@@ -334,7 +353,7 @@ class AsyncSocket(object):
             try:
                 rv = self.socket.connect_ex((address, port))
             except socket.gaierror:
-                trapCall(self, errback, ConnectionError('gaierror'))
+                trap_call(self, errback, ConnectionError('gaierror'))
                 return
             if rv in (0, errno.EINPROGRESS, errno.EWOULDBLOCK):
                 eventloop.addWriteCallback(self.socket, onWriteReady)
@@ -346,20 +365,20 @@ class AsyncSocket(object):
                     msg = errno.errorcode[rv]
                 except KeyError:
                     msg = "Unknown connection error: %s" % rv
-                trapCall(self, errback, ConnectionError(msg))
+                trap_call(self, errback, ConnectionError(msg))
         def onWriteReady():
             eventloop.removeWriteCallback(self.socket)
             self.socketConnectTimeout.cancel()
             rv = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
             if rv == 0:
-                trapCall(self, callback, self)
+                trap_call(self, callback, self)
             else:
                 msg = errno.errorcode.get(rv, _('Unknown Error code'))
-                trapCall(self, errback, ConnectionError(msg))
+                trap_call(self, errback, ConnectionError(msg))
             self.connectionErrback = None
         def onWriteTimeout():
             eventloop.removeWriteCallback(self.socket)
-            trapCall(self, errback, ConnectionTimeout(host))
+            trap_call(self, errback, ConnectionTimeout(host))
             self.connectionErrback = None
         eventloop.callInThread(onAddressLookup, handleGetHostByNameException,
                 socket.gethostbyname, "getHostByName - %s" % host, host)
@@ -368,13 +387,13 @@ class AsyncSocket(object):
         def finishAccept():
             eventloop.removeReadCallback(self.socket)
             (self.socket, addr) = self.socket.accept()
-            trapCall(self, callback, self)
+            trap_call(self, callback, self)
             self.connectionErrback = None
 
         self.name = "Incoming %s:%s" % (host, port)
         self.connectionErrback = errback
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.socket.bind( (host, port) )
+        self.socket.bind((host, port))
         (self.addr, self.port) = self.socket.getsockname()
         self.socket.listen(63)
         eventloop.addReadCallback(self.socket, finishAccept)
@@ -387,13 +406,13 @@ class AsyncSocket(object):
             self.socket = None
             if self.connectionErrback is not None:
                 error = NetworkError(_("Connection closed"))
-                trapCall(self, self.connectionErrback, error)
+                trap_call(self, self.connectionErrback, error)
                 self.connectionErrback = None
 
     def isOpen(self):
         return self.socket is not None
 
-    def sendData(self, data, callback = None):
+    def sendData(self, data, callback=None):
         """Send data out to the socket when it becomes ready.
         
         NOTE: currently we have no way of detecting when the data gets sent
@@ -497,10 +516,10 @@ class AsyncSocket(object):
         self.startReadTimeout()
         if data == '':
             if self.closeCallback:
-                trapCall(self, self.closeCallback, self, socket.SHUT_RD)
+                trap_call(self, self.closeCallback, self, socket.SHUT_RD)
         else:
             self.readSomeData = True
-            trapCall(self, self.readCallback, data)
+            trap_call(self, self.readCallback, data)
 
     def handleEarlyClose(self, operation):
         self.closeConnection()
@@ -509,14 +528,14 @@ class AsyncSocket(object):
                 type = socket.SHUT_RD
             else:
                 type = socket.SHUT_WR
-            trapCall(self, self.closeCallback, self, type)
+            trap_call(self, self.closeCallback, self, type)
 
 class AsyncSSLStream(AsyncSocket):
     def __init__(self, closeCallback=None):
         super(AsyncSSLStream, self).__init__(closeCallback)
         self.interruptedOperation = None
 
-    def openConnection(self, host, port, callback, errback, disableReadTimeout = None):
+    def openConnection(self, host, port, callback, errback, disableReadTimeout=None):
         def onSocketOpen(self):
             self.socket.setblocking(1)
             eventloop.callInThread(onSSLOpen, handleSSLError, socket.ssl,
@@ -598,11 +617,7 @@ class ProxiedAsyncSSLStream(AsyncSSLStream):
             eventloop.callInThread(onSSLOpen, handleSSLError, lambda: openProxyConnection(self),
                                    "ProxiedAsyncSSL openProxyConnection()")
         def openProxyConnection(self):
-            headers = {'User-Agent': '%s/%s (%s)' % (
-                config.get(prefs.SHORT_APP_NAME),
-                config.get(prefs.APP_VERSION),
-                config.get(prefs.PROJECT_URL)),
-                "Host": host}
+            headers = {'User-Agent': user_agent(), "Host": host}
             if config.get(prefs.HTTP_PROXY_AUTHORIZATION_ACTIVE):
                 username = config.get(prefs.HTTP_PROXY_AUTHORIZATION_USERNAME)
                 password = config.get(prefs.HTTP_PROXY_AUTHORIZATION_PASSWORD)
@@ -621,7 +636,7 @@ class ProxiedAsyncSSLStream(AsyncSSLStream):
                 while (data.find("\r\n\r\n") == -1):
                     data += self.socket.recv(1)
                 data = data.split("\r\n")
-                if (-1 == data[0].find(' 200 ')):                   
+                if -1 == data[0].find(' 200 '):
                     eventloop.addIdle(lambda :handleSSLError(NetworkError(data[0])),"Network Error")
                 else:
                     return socket.ssl(self.socket)
@@ -678,7 +693,7 @@ class ConnectionHandler(object):
         self.port = port
         def callbackIntercept(asyncSocket):
             if callback:
-                trapCall(self, callback, self)
+                trap_call(self, callback, self)
         self.stream.openConnection(host, port, callbackIntercept, errback, disableReadTimeout)
 
     def closeConnection(self):
@@ -686,7 +701,7 @@ class ConnectionHandler(object):
             self.stream.closeConnection()
         self.changeState('closed')
 
-    def sendData(self, data, callback = None):
+    def sendData(self, data, callback=None):
         self.stream.sendData(data, callback)
 
     def changeState(self, newState):
@@ -767,7 +782,7 @@ class HTTPConnection(ConnectionHandler):
 
         if self.pipelinedRequest is not None:
             errback = self.pipelinedRequest[1]
-            trapCall(self, errback, PipelinedRequestNeverStarted())
+            trap_call(self, errback, PipelinedRequestNeverStarted())
             self.pipelinedRequest = None
 
     def canSendRequest(self):
@@ -777,8 +792,8 @@ class HTTPConnection(ConnectionHandler):
 
     def sendRequest(self, callback, errback, host, port,
                     requestStartCallback=None, headerCallback=None,
-                    bodyDataCallback = None, method="GET", path='/',
-                    headers=None, postVariables = None, postFiles = None):
+                    bodyDataCallback=None, method="GET", path='/',
+                    headers=None, postVariables=None, postFiles=None):
         """Sending an HTTP Request.  callback will be called if the request
         completes normally, errback will be called if there is a network
         error.
@@ -825,11 +840,11 @@ class HTTPConnection(ConnectionHandler):
 
         if (method == "POST" and postVariables is not None and
                             len(postVariables) > 0 and postFiles is None):
-            postData = URLEncodeDict(postVariables)
+            postData = url_encode_dict(postVariables)
             headers['Content-Type'] = 'application/x-www-form-urlencoded'
             headers['Content-Length'] = '%d' % len(postData)
         elif method == "POST" and postFiles is not None:
-            (postData, boundary) = multipartEncode(postVariables, postFiles)
+            (postData, boundary) = multipart_encode(postVariables, postFiles)
             headers['Content-Type'] = 'multipart/form-data; boundary=%s' % boundary
             headers['Content-Length'] = '%d' % len(postData)
         else:
@@ -851,7 +866,7 @@ class HTTPConnection(ConnectionHandler):
         """
 
         if requestStartCallback:
-            trapCall(self, requestStartCallback, self)
+            trap_call(self, requestStartCallback, self)
             if self.state == 'closed':
                 return
 
@@ -874,7 +889,7 @@ class HTTPConnection(ConnectionHandler):
         self.sentReadyCallback = False
         self.changeState('response-status')
 
-    def sendRequestData(self, method, path, headers, data = None):
+    def sendRequestData(self, method, path, headers, data=None):
         sendOut = []
         path = path.encode("ascii", "replace")
         path = urllib.quote(path, safe="-_.!~*'();/?:@&=+$,%#")
@@ -914,7 +929,7 @@ class HTTPConnection(ConnectionHandler):
             if data == '':
                 return
             self.bodyBytesRead += len(data)
-            trapCall(self, self.bodyDataCallback, data)
+            trap_call(self, self.bodyDataCallback, data)
             if self.state == 'closed':
                 return 
             if (self.contentLength is not None and 
@@ -947,7 +962,7 @@ class HTTPConnection(ConnectionHandler):
             self.chunkBytesRead += len(data)
             if data == '':
                 return
-            trapCall(self, self.bodyDataCallback, data)
+            trap_call(self, self.bodyDataCallback, data)
             if self.chunkBytesRead == self.chunkSize:
                 self.changeState('chunk-crlf')
         elif self.buffer.length >= self.chunkSize:
@@ -1050,10 +1065,10 @@ class HTTPConnection(ConnectionHandler):
         self.checkChunked()
         self.decideWillClose()
         if self.headerCallback:
-            trapCall(self, self.headerCallback, self.makeResponse())
+            trap_call(self, self.headerCallback, self.makeResponse())
         if self.state == 'closed':
             return # maybe the header callback cancelled this request
-        if ((100 <= self.status <= 199) or self.status in (204, 304) or
+        if ((100 <= self.status <= 199) or self.status in (204, 301, 303, 304, 307) or
                 self.method == 'HEAD' or self.contentLength == 0):
             self.finishRequest()
         else:
@@ -1128,7 +1143,7 @@ class HTTPConnection(ConnectionHandler):
             else:
                 self.changeState('ready')
                 self.idleSince = clock()
-        trapCall(self, origCallback, response)
+        trap_call(self, origCallback, response)
         self.requestsFinished += 1
         self.maybeSendReadyCallback()
 
@@ -1140,15 +1155,14 @@ class HTTPConnection(ConnectionHandler):
             response[key] = getattr(self, key)
         return response
 
+    # This needs to be in an idle so that the connection is added
+    # to the "active" list before the open callback happens --NN
+    @eventloop.asIdle
     def maybeSendReadyCallback(self):
         if (self.readyCallback and self.canSendRequest() and not
                 self.sentReadyCallback):
             self.sentReadyCallback = True
-
-            # This needs to be in an idle so that the connection is added
-            # to the "active" list before the open callback happens --NN
-            eventloop.addIdle(lambda : self.readyCallback(self),
-                              "Ready Callback %s" % str(self))
+            self.readyCallback(self)
         
     def handleClose(self, type):
         oldState = self.state
@@ -1164,7 +1178,7 @@ class HTTPConnection(ConnectionHandler):
 
     def handleError(self, error):
         self.closeConnection()
-        trapCall(self, self.errback, error)
+        trap_call(self, self.errback, error)
 
 class HTTPSConnection(HTTPConnection):
     streamFactory = AsyncSSLStream
@@ -1268,8 +1282,6 @@ class HTTPConnectionPool(object):
             postVariables = None, postFiles = None):
         """Add a request to be run.  The request will run immediately if we
         have a free connection, otherwise it will be queued.
-
-        returns a request id that can be passed to cancelRequest
         """
         proxy_host = proxy_port = None
         scheme, host, port, path = parseURL(url)
@@ -1402,7 +1414,6 @@ class HTTPConnectionPool(object):
         return conn
 
     def _dropAFreeConnection(self):
-        # TODO: pick based on LRU
         firstTime = sys.maxint
         toDrop = None
 
@@ -1410,6 +1421,7 @@ class HTTPConnectionPool(object):
             for candidate in conns['free']:
                 if candidate.idleSince < firstTime:
                     toDrop = candidate
+                    firstTime = candidate.idleSince
         if toDrop is not None:
             toDrop.closeConnection()
 
@@ -1443,7 +1455,7 @@ class HTTPClient(object):
 
     def __init__(self, url, callback, errback, headerCallback=None,
             bodyDataCallback=None, method="GET", start=0, etag=None,
-            modified=None, cookies=None, postVariables = None, postFiles = None):
+            modified=None, cookies=None, postVariables=None, postFiles=None):
         if cookies == None:
             cookies = {}
         
@@ -1470,10 +1482,7 @@ class HTTPClient(object):
         self.authAttempts = 0
         self.updateURLOk = True
         self.originalURL = self.updatedURL = self.redirectedURL = url
-        self.userAgent = "%s/%s (%s)" % \
-                         (config.get(prefs.SHORT_APP_NAME),
-                          config.get(prefs.APP_VERSION),
-                          config.get(prefs.PROJECT_URL))
+        self.user_agent = user_agent()
         self.connection = None
         self.cancelled = False
         self.initHeaders()
@@ -1530,7 +1539,7 @@ class HTTPClient(object):
             self.headers["If-None-Match"] = self.etag
         if not self.modified is None:
             self.headers["If-Modified-Since"] = self.modified
-        self.headers['User-Agent'] = self.userAgent
+        self.headers['User-Agent'] = self.user_agent
         self.setCookieHeader()
 
     def setCookieHeader(self):
@@ -1563,7 +1572,10 @@ class HTTPClient(object):
                 if authHeader is not None:
                     self.headers["Authorization"] = authHeader
                 self.reallyStartRequest()
-            httpauth.findHTTPAuth(callback, host.decode('ascii','replace'), path.decode('ascii','replace'))
+            try:
+                httpauth.find_http_auth(callback, host.decode('ascii', 'replace'), path.decode('ascii', 'replace'))
+            except UnicodeDecodeError:
+                httpauth.find_http_auth(callback, host.decode('ascii', 'ignore'), path.decode('ascii', 'ignore'))
         else:
             self.reallyStartRequest()
 
@@ -1603,7 +1615,7 @@ class HTTPClient(object):
             if not self.gotBadStatusCode:
                 if self.callback:
                     response = self.prepareResponse(response)
-                    trapCall(self, self.callback, response)
+                    trap_call(self, self.callback, response)
             elif self.errback:
                 error = UnexpectedStatusCode(response['status'])
                 self.errbackIntercept(error)
@@ -1625,7 +1637,7 @@ class HTTPClient(object):
             self.startRequest()
         else:
             self.connection = None
-            trapCall(self, self.errback, error)
+            trap_call(self, self.errback, error)
 
     def onRequestStart(self, connection):
         if self.cancelled:
@@ -1641,13 +1653,13 @@ class HTTPClient(object):
                 self.gotBadStatusCode = True
             if self.headerCallback is not None:
                 response = self.prepareResponse(response)
-                if not trapCall(self, self.headerCallback, response):
+                if not trap_call(self, self.headerCallback, response):
                     self.cancel()
 
     def onBodyData(self, data):
         if (not self.willHandleResponse and not self.gotBadStatusCode and 
                 self.bodyDataCallback):
-            if not trapCall(self, self.bodyDataCallback, data):
+            if not trap_call(self, self.bodyDataCallback, data):
                 self.cancel()
 
     def prepareResponse(self, response):
@@ -1658,6 +1670,8 @@ class HTTPClient(object):
         response['charset'] = self.getCharsetFromResponse(response)
         try:
             response['cookies'] = self.getCookiesFromResponse(response)
+        except (SystemExit, KeyboardInterrupt):
+            raise
         except:
             print "ERROR in getCookiesFromResponse()"
             traceback.print_exc()
@@ -1829,12 +1843,12 @@ class HTTPClient(object):
         match = re.search("(\w+)\s+realm\s*=\s*\"(.*?)\"$",
             response['www-authenticate'])
         if match is None:
-            trapCall(self, self.errback, AuthorizationFailed())
+            trap_call(self, self.errback, AuthorizationFailed())
             return
         authScheme = unicode(match.expand("\\1"))
         realm = unicode(match.expand("\\2"))
         if authScheme.lower() != 'basic':
-            trapCall(self, self.errback, AuthorizationFailed())
+            trap_call(self, self.errback, AuthorizationFailed())
             return
         def callback(authHeader):
             if authHeader is not None:
@@ -1842,15 +1856,15 @@ class HTTPClient(object):
                 self.authAttempts += 1
                 self.startRequest()
             else:
-                trapCall(self, self.errback, AuthorizationFailed())
+                trap_call(self, self.errback, AuthorizationFailed())
         httpauth.askForHTTPAuth(callback, self.url, realm, authScheme)
 
 # Grabs a URL in the background using the eventloop
 # defaultMimeType is used for file:// URLs
 def grabURL(url, callback, errback, headerCallback=None,
         bodyDataCallback=None, method="GET", start=0, etag=None,
-        modified=None, cookies=None, postVariables = None, postFiles = None,
-        defaultMimeType='application/octet-stream', clientClass = HTTPClient):
+        modified=None, cookies=None, postVariables=None, postFiles=None,
+        defaultMimeType='application/octet-stream', clientClass=HTTPClient):
     if cookies == None:
         cookies = {}
     if url.startswith("file://"):
@@ -1862,6 +1876,8 @@ def grabURL(url, callback, errback, headerCallback=None,
         else:
             try:
                 data = f.read()
+            except (SystemExit, KeyboardInterrupt):
+                raise
             except:
                 errback(FileURLReadError(path))
             else:
@@ -1912,7 +1928,7 @@ class HTTPHeaderGrabber(HTTPClient):
             self.callback(self.prepareResponse(headers))
             self.cancel()
 
-def grabHeaders (url, callback, errback,  clientClass = HTTPHeaderGrabber):
+def grabHeaders(url, callback, errback,  clientClass=HTTPHeaderGrabber):
     client = clientClass(url, callback, errback)
     client.startRequest()
     return client
